"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Filler = void 0;
const random_js_1 = require("random-js");
const generators_1 = require("./generators");
class Filler {
    constructor(dbConnector, schema, callback = () => null) {
        this.dbConnector = dbConnector;
        this.schema = schema;
        this.callback = callback;
        this.continue = false;
        if (schema.settings.seed) {
            this.random = new random_js_1.Random(random_js_1.MersenneTwister19937.seed(schema.settings.seed));
        }
        else {
            this.random = new random_js_1.Random(random_js_1.MersenneTwister19937.autoSeed());
        }
    }
    gotoNextTable() {
        this.continue = true;
    }
    empty(table) {
        return __awaiter(this, void 0, void 0, function* () {
            this.callback({ currentTable: table.name, step: 'empty', state: 'RUNNING', currentValue: 0, max: 1 });
            yield this.dbConnector.emptyTable(table);
            this.callback({ currentTable: table.name, step: 'empty', state: 'DONE', currentValue: 1, max: 1 });
        });
    }
    fillTables(reset = false) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.beforeAll();
            for (const table of this.schema.tables) {
                yield this.fill(table, reset);
            }
            yield this.afterAll();
        });
    }
    fill(table, reset = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (reset)
                yield this.empty(table);
            const handleTriggers = table.disableTriggers || (table.disableTriggers === undefined && this.schema.settings.disableTriggers);
            if (handleTriggers)
                yield this.dbConnector.disableTriggers(table.name);
            yield this.before(table);
            const insertedRows = yield this.generateData(table);
            if (insertedRows)
                yield this.after(table);
            if (handleTriggers)
                yield this.dbConnector.enableTriggers(table.name);
        });
    }
    beforeAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.schema.settings.beforeAll.length === 0)
                return;
            for (let i = 0; i < this.schema.settings.beforeAll.length; i++) {
                this.callback({ currentTable: '', step: 'beforeAll', state: 'RUNNING', currentValue: i, max: this.schema.settings.beforeAll.length });
                yield this.dbConnector.executeRawQuery(this.schema.settings.beforeAll[i]);
            }
            this.callback({ currentTable: '', step: 'beforeAll', state: 'DONE', currentValue: this.schema.settings.beforeAll.length, max: this.schema.settings.beforeAll.length });
        });
    }
    afterAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.schema.settings.afterAll.length === 0)
                return;
            for (let i = 0; i < this.schema.settings.afterAll.length; i++) {
                this.callback({ currentTable: '', step: 'afterAll', state: 'RUNNING', currentValue: i, max: this.schema.settings.afterAll.length });
                yield this.dbConnector.executeRawQuery(this.schema.settings.afterAll[i]);
            }
            this.callback({ currentTable: '', step: 'afterAll', state: 'DONE', currentValue: this.schema.settings.afterAll.length, max: this.schema.settings.afterAll.length });
        });
    }
    before(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (table.before.length === 0)
                return;
            for (let i = 0; i < table.before.length; i++) {
                this.callback({ currentTable: table.name, state: 'RUNNING', step: 'before', currentValue: i, max: table.before.length });
                yield this.dbConnector.executeRawQuery(table.before[i]);
            }
            this.callback({ currentTable: table.name, step: 'before', state: 'DONE', currentValue: table.after.length, max: table.after.length });
        });
    }
    generateData(table) {
        return __awaiter(this, void 0, void 0, function* () {
            let previousRunRows = -1;
            let currentNbRows = yield this.dbConnector.countLines(table);
            let maxLines = 0;
            if (table.addLines !== undefined) {
                maxLines = currentNbRows + table.addLines;
                if (table.maxLines !== undefined)
                    maxLines = Math.min(maxLines, table.maxLines);
            }
            else if (table.maxLines !== undefined)
                maxLines = table.maxLines;
            let insertedRows = 0;
            table.deltaRows = maxLines - currentNbRows;
            if (table.deltaRows <= 0)
                return 0;
            let currentTableRow = 0;
            const generatorBuilder = new generators_1.GeneratorBuilder(this.random, this.dbConnector, table);
            const generators = yield Promise.all(table.columns
                .map((column) => generatorBuilder.build(column))
                .map((g) => g.init()));
            this.callback({ currentTable: table.name, step: 'generateData', state: 'RUNNING', currentValue: currentNbRows, max: maxLines });
            TABLE_LOOP: while (currentNbRows < maxLines) {
                previousRunRows = currentNbRows;
                const rows = [];
                const runRows = Math.min(this.schema.settings.maxRowsPerBatch, maxLines - currentNbRows);
                BATCH_LOOP: for (let currentBatchRow = 0; currentBatchRow < runRows; currentBatchRow++) {
                    const row = {};
                    for (let c = 0; c < table.columns.length; c++) {
                        const column = table.columns[c];
                        if (column.autoIncrement)
                            continue;
                        try {
                            row[column.name] = generators[c].generate(currentTableRow, row);
                        }
                        catch (ex) {
                            this.callback({ currentTable: table.name, step: 'generateData', state: 'RUNNING', currentValue: currentNbRows, max: maxLines, comment: ex.message + '. ' });
                            break BATCH_LOOP;
                        }
                        if (column.nullable > 0 && this.random.real(0, 1) < column.nullable)
                            row[column.name] = null;
                    }
                    rows.push(row);
                    currentTableRow++;
                }
                insertedRows = yield this.dbConnector.insert(table.name, rows);
                currentNbRows += insertedRows;
                if (previousRunRows === currentNbRows) {
                    this.callback({ currentTable: table.name, step: 'generateData', state: 'DONE', currentValue: currentNbRows, max: maxLines, comment: 'Last run did not insert any rows. ' });
                    break TABLE_LOOP;
                }
                this.callback({ currentTable: table.name, step: 'generateData', state: 'RUNNING', currentValue: currentNbRows, max: maxLines });
                if (this.continue) {
                    this.continue = false;
                    break TABLE_LOOP;
                }
            }
            this.callback({ currentTable: table.name, step: 'generateData', state: 'DONE', currentValue: currentNbRows, max: maxLines });
            return insertedRows;
        });
    }
    after(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (table.after.length === 0)
                return;
            for (let i = 0; i < table.after.length; i++) {
                this.callback({ currentTable: table.name, step: 'after', state: 'RUNNING', currentValue: i, max: table.after.length });
                yield this.dbConnector.executeRawQuery(table.after[i]);
            }
            this.callback({ currentTable: table.name, step: 'after', state: 'DONE', currentValue: table.after.length, max: table.after.length });
        });
    }
}
exports.Filler = Filler;
