"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomizedForeignKey = exports.CustomizedColumn = exports.CustomizedTable = exports.CustomizedSchema = void 0;
const builder_1 = require("../builder");
const generators_1 = require("../generation/generators");
const generators_2 = require("../generation/generators/generators");
const custom_schema_class_1 = require("./custom-schema.class");
const schema_class_1 = require("./schema.class");
class CustomizedSchema extends custom_schema_class_1.CustomSchema {
    constructor() {
        super(...arguments);
        this.tables = [];
    }
    static create(schema, customSchema = new custom_schema_class_1.CustomSchema()) {
        const customizedSchema = new CustomizedSchema();
        customizedSchema.settings = Object.assign({}, customizedSchema.settings, customSchema.settings);
        const tables = schema.tables
            .filter((table) => {
            return !customSchema.settings.ignoredTables.includes(table.name)
                && (customSchema.settings.tablesToFill.length === 0
                    || customSchema.settings.tablesToFill.includes(table.name));
        });
        const customizedTables = tables.map((table) => {
            const customizedTable = this.customizeTable(table, customSchema);
            return customizedTable;
        });
        customizedSchema.tables = this.orderTablesByForeignKeys(customizedTables);
        return customizedSchema;
    }
    static parseValues(columnValues, globalValues) {
        let parsedValues = [];
        let values = columnValues;
        if (typeof values === 'string') {
            values = globalValues[values];
        }
        if (!(values instanceof Array)) {
            Object.keys(values).forEach((key) => {
                let arr = new Array(Math.round(values[key] * 100));
                arr = arr.fill(key);
                parsedValues = parsedValues.concat(arr);
            });
        }
        else {
            parsedValues = values;
        }
        if (parsedValues.length === 0)
            throw new Error(`No values found`);
        return parsedValues;
    }
    static customizeTable(table, customSchema) {
        const customizedTable = new CustomizedTable();
        customizedTable.name = table.name;
        customizedTable.referencedTables = table.referencedTables;
        const customTable = customSchema.tables.find(t => t.name && t.name.toLowerCase() === table.name.toLowerCase());
        if (customTable) {
            customizedTable.maxLines = (customTable.maxLines !== undefined ? customTable.maxLines : Infinity);
            customizedTable.addLines = (customTable.addLines !== undefined ? customTable.addLines : Infinity);
            customizedTable.disableTriggers = customTable.disableTriggers || false;
            customizedTable.before = customTable.before || [];
            customizedTable.after = customTable.after || [];
        }
        customizedTable.columns = table.columns.map((column) => {
            var _a;
            const globalSetting = customSchema.settings.options.find(o => o.generators.includes(column.generator));
            let customColumn = (_a = customTable === null || customTable === void 0 ? void 0 : customTable.columns) === null || _a === void 0 ? void 0 : _a.find(c => c.name.toLowerCase() === column.name.toLowerCase());
            customColumn = Object.assign({}, column, globalSetting === null || globalSetting === void 0 ? void 0 : globalSetting.options, customColumn);
            const customizedColumnBuilder = new builder_1.Builder(CustomizedColumn);
            if (customColumn.autoIncrement !== undefined)
                customizedColumnBuilder.set('autoIncrement', customColumn.autoIncrement);
            if (customColumn.generator !== undefined)
                customizedColumnBuilder.set('generator', customColumn.generator);
            if (customColumn.max !== undefined)
                customizedColumnBuilder.set('max', customColumn.max);
            if (customColumn.min !== undefined)
                customizedColumnBuilder.set('min', customColumn.min);
            if (customColumn.maxDate !== undefined)
                customizedColumnBuilder.set('maxDate', customColumn.maxDate);
            if (customColumn.minDate !== undefined)
                customizedColumnBuilder.set('minDate', customColumn.minDate);
            if (customColumn.monotonic !== undefined)
                customizedColumnBuilder.set('monotonic', customColumn.monotonic);
            if (customColumn.name !== undefined)
                customizedColumnBuilder.set('name', customColumn.name);
            if (customColumn.nullable !== undefined)
                customizedColumnBuilder.set('nullable', customColumn.nullable);
            if (customColumn.unique !== undefined)
                customizedColumnBuilder.set('unique', customColumn.unique);
            if (customColumn.unsigned !== undefined)
                customizedColumnBuilder.set('unsigned', customColumn.unsigned);
            if (customColumn.customFunction !== undefined)
                customizedColumnBuilder.set('customFunction', customColumn.customFunction);
            if (customColumn === null || customColumn === void 0 ? void 0 : customColumn.foreignKey) {
                customizedColumnBuilder.set('foreignKey', customColumn.foreignKey);
                customizedTable.referencedTables.push(customColumn.foreignKey.table);
            }
            if (customColumn.generator === generators_2.Generators.values)
                customizedColumnBuilder.set('values', CustomizedSchema.parseValues(customColumn.values || [], customSchema.settings.values));
            if (customColumn.generator === generators_2.Generators.string) {
                if (customColumn.max !== undefined && customSchema.settings.maxLengthValue !== undefined && customColumn.max > customSchema.settings.maxLengthValue) {
                    customizedColumnBuilder.set('max', customSchema.settings.maxLengthValue);
                }
            }
            const customizedColumn = customizedColumnBuilder.build();
            generators_1.GeneratorBuilder.validate(customizedTable, customizedColumn);
            return customizedColumn;
        }).sort((c1, c2) => {
            if (!customTable)
                return 0;
            if (!customTable.columns)
                return 0;
            const cc1 = customTable.columns.findIndex(c => c.name.toLowerCase() === c1.name.toLowerCase());
            const cc2 = customTable.columns.findIndex(c => c.name.toLowerCase() === c2.name.toLowerCase());
            if (cc1 === -1 && cc2 >= 0)
                return -1;
            if (cc1 === -1 && cc2 === -1)
                return 0;
            if (cc1 >= 0 && cc2 === -1)
                return 1;
            if (cc1 < cc2)
                return -1;
            return 1;
        });
        return customizedTable;
    }
    static orderTablesByForeignKeys(tables) {
        const sortedTables = [];
        const recursive = (branch) => {
            const table = branch[branch.length - 1];
            while (table.referencedTables.length > 0) {
                const tableName = table.referencedTables.pop();
                const referencedTable = tables.find((t) => {
                    return t.name === tableName;
                });
                if (referencedTable)
                    recursive([].concat(branch, referencedTable));
            }
            ;
            if (sortedTables.find((t) => t.name.toLowerCase() === table.name.toLowerCase()))
                return;
            sortedTables.push({
                name: table.name,
                disableTriggers: table.disableTriggers,
                maxLines: table.maxLines,
                addLines: table.addLines,
                deltaRows: table.addLines || table.maxLines,
                columns: table.columns,
                before: table.before,
                after: table.after,
                referencedTables: [],
            });
            branch.pop();
            return;
        };
        tables.forEach((table) => {
            recursive([table]);
        });
        return sortedTables;
    }
    ;
}
exports.CustomizedSchema = CustomizedSchema;
class CustomizedTable {
    constructor() {
        this.name = '';
        this.columns = [];
        this.referencedTables = [];
        this.before = [];
        this.after = [];
        this.maxLines = 1000;
        this.addLines = Infinity;
        this.deltaRows = 0;
        this.disableTriggers = false;
    }
}
exports.CustomizedTable = CustomizedTable;
class CustomizedColumn extends schema_class_1.Column {
}
exports.CustomizedColumn = CustomizedColumn;
class CustomizedForeignKey extends schema_class_1.ForeignKey {
}
exports.CustomizedForeignKey = CustomizedForeignKey;
